@@ -1,4 +1,4 @@
-"""Downloading utilities for urls"""
+"""Downloading utilities for urls."""
 import hashlib
 import logging
 from copy import deepcopy
@@ -25,8 +25,9 @@ logger = logging.getLogger(__name__)
 
 
 class Download(BaseDownload):
-    """Download class with various download operations. Requires either global user agent to be set or appropriate
-    user agent parameter(s) to be completed.
+    """Download class with various download operations. Requires either global
+    user agent to be set or appropriate user agent parameter(s) to be
+    completed.
 
     Args:
         user_agent (Optional[str]): User agent string. HDXPythonUtilities/X.X.X- is prefixed.
@@ -34,6 +35,7 @@ class Download(BaseDownload):
         user_agent_lookup (Optional[str]): Lookup key for YAML. Ignored if user_agent supplied.
         use_env (bool): Whether to read environment variables. Defaults to True.
         fail_on_missing_file (bool): Raise an exception if any specified configuration files are missing. Defaults to True.
+        verify (bool): Whether to verify SSL certificates. Defaults to True.
         rate_limit (Optional[Dict]): Rate limiting per host eg. {"calls": 1, "period": 0.1}. Defaults to None.
         **kwargs: See below
         auth (Tuple[str, str]): Authorisation information in tuple form (user, pass) OR
@@ -48,7 +50,7 @@ class Download(BaseDownload):
         allowed_methods (iterable): HTTP methods for which to force retry. Defaults t0 frozenset(['GET']).
     """
 
-    downloaders = dict()
+    downloaders = {}
 
     def __init__(
         self,
@@ -57,6 +59,7 @@ class Download(BaseDownload):
         user_agent_lookup: Optional[str] = None,
         use_env: bool = True,
         fail_on_missing_file: bool = True,
+        verify: bool = True,
         rate_limit: Optional[Dict] = None,
         **kwargs: Any,
     ) -> None:
@@ -70,6 +73,7 @@ class Download(BaseDownload):
                 user_agent_lookup,
                 use_env,
                 fail_on_missing_file,
+                verify,
                 **kwargs,
             )
         self.response = None
@@ -83,11 +87,10 @@ class Download(BaseDownload):
             self.setup = self.normal_setup
 
     def close_response(self) -> None:
-        """Close response
+        """Close response.
 
         Returns:
             None
-
         """
         if self.response:
             try:
@@ -96,18 +99,16 @@ class Download(BaseDownload):
                 pass
 
     def close(self) -> None:
-        """Close response and session
+        """Close response and session.
 
         Returns:
             None
-
         """
         self.close_response()
         self.session.close()
 
     def __exit__(self, exc_type: Any, exc_value: Any, traceback: Any) -> None:
-        """
-        Allow usage of with
+        """Allow usage of with.
 
         Args:
             exc_type (Any): Exception type
@@ -116,7 +117,6 @@ class Download(BaseDownload):
 
         Returns:
             None
-
         """
         self.close()
 
@@ -127,8 +127,10 @@ class Download(BaseDownload):
         filename: Optional[str] = None,
         path: Optional[str] = None,
         overwrite: bool = False,
+        keep: bool = False,
     ) -> str:
-        """Get filename from url and join to provided folder or temporary folder if no folder supplied, ensuring uniqueness
+        """Get filename from url and join to provided folder or temporary
+        folder if no folder supplied, ensuring uniqueness.
 
         Args:
             url (str): URL to download
@@ -136,10 +138,10 @@ class Download(BaseDownload):
             filename (Optional[str]): Filename to use for downloaded file. Defaults to None (derive from the url).
             path (Optional[str]): Full path to use for downloaded file. Defaults to None (use folder and filename).
             overwrite (bool): Whether to overwrite existing file. Defaults to False.
+            keep (bool): Whether to keep already downloaded file. Defaults to False.
 
         Returns:
             str: Path of downloaded file
-
         """
         if path:
             if folder or filename:
@@ -158,7 +160,7 @@ class Download(BaseDownload):
                 remove(path)
             except OSError:
                 pass
-        else:
+        elif not keep:
             count = 0
             while exists(path):
                 count += 1
@@ -166,7 +168,7 @@ class Download(BaseDownload):
         return path
 
     def get_full_url(self, url: str) -> str:
-        """Get full url including any additional parameters
+        """Get full url including any additional parameters.
 
         Args:
             url (str): URL for which to get full url
@@ -180,7 +182,7 @@ class Download(BaseDownload):
 
     @staticmethod
     def get_url_for_get(url: str, parameters: Optional[Dict] = None) -> str:
-        """Get full url for GET request including parameters
+        """Get full url for GET request including parameters.
 
         Args:
             url (str): URL to download
@@ -188,7 +190,6 @@ class Download(BaseDownload):
 
         Returns:
             str: Full url
-
         """
         spliturl = urlsplit(url)
         getparams = dict(parse_qsl(spliturl.query))
@@ -201,7 +202,8 @@ class Download(BaseDownload):
     def get_url_params_for_post(
         url: str, parameters: Optional[Dict] = None
     ) -> Tuple[str, Dict]:
-        """Get full url for POST request and all parameters including any in the url
+        """Get full url for POST request and all parameters including any in
+        the url.
 
         Args:
             url (str): URL to download
@@ -209,7 +211,6 @@ class Download(BaseDownload):
 
         Returns:
             Tuple[str, Dict]: (Full url, parameters)
-
         """
         spliturl = urlsplit(url)
         getparams = dict(parse_qsl(spliturl.query))
@@ -225,8 +226,9 @@ class Download(BaseDownload):
         hxltags: Dict[str, str],
         dict_form: bool = False,
     ) -> Union[List[str], Dict[str, str]]:
-        """Return HXL tag row for header row given list of headers and dictionary with header to HXL hashtag mappings.
-        Return list or dictionary depending upon the dict_form argument.
+        """Return HXL tag row for header row given list of headers and
+        dictionary with header to HXL hashtag mappings. Return list or
+        dictionary depending upon the dict_form argument.
 
         Args:
             headers (ListTuple[str]): Headers for which to get HXL hashtags
@@ -235,12 +237,10 @@ class Download(BaseDownload):
 
         Returns:
             Union[List[str],Dict[str,str]]: Return either a list or dictionary conating HXL hashtags
-
         """
         if dict_form:
             return {header: hxltags.get(header, "") for header in headers}
-        else:
-            return [hxltags.get(header, "") for header in headers]
+        return [hxltags.get(header, "") for header in headers]
 
     def normal_setup(
         self,
@@ -252,7 +252,7 @@ class Download(BaseDownload):
         headers: Optional[Dict] = None,
         encoding: Optional[str] = None,
     ) -> requests.Response:
-        """Setup download from provided url returning the response
+        """Setup download from provided url returning the response.
 
         Args:
             url (str): URL or path to download
@@ -265,7 +265,6 @@ class Download(BaseDownload):
 
         Returns:
             requests.Response: requests.Response object
-
         """
         self.close_response()
         self.response = None
@@ -305,14 +304,14 @@ class Download(BaseDownload):
         return self.response
 
     def hash_stream(self, url: str) -> str:
-        """Stream file from url and hash it using MD5. Must call setup method first.
+        """Stream file from url and hash it using MD5. Must call setup method
+        first.
 
         Args:
             url (str): URL or path to download
 
         Returns:
             str: MD5 hash of file
-
         """
         md5hash = hashlib.md5()
         try:
@@ -325,29 +324,17 @@ class Download(BaseDownload):
                 f"Download of {url} failed in retrieval of stream!" % url
             )
 
-    def stream_file(
-        self,
-        url: str,
-        folder: Optional[str] = None,
-        filename: Optional[str] = None,
-        path: Optional[str] = None,
-        overwrite: bool = False,
-    ) -> str:
-        """Stream file from url and store in provided folder or temporary folder if no folder supplied.
-        Must call setup method first.
+    def stream_path(self, path: str, errormsg: str):
+        """Stream file from url and store in provided path. Must call setup
+        method first.
 
         Args:
-            url (str): URL or path to download
-            folder (Optional[str]): Folder to download it to. Defaults to None (temporary folder).
-            filename (Optional[str]): Filename to use for downloaded file. Defaults to None (derive from the url).
-            path (Optional[str]): Full path to use for downloaded file. Defaults to None (use folder and filename).
-            overwrite (bool): Whether to overwrite existing file. Defaults to False.
+            path (str): Path for downloaded file
+            errormsg (str): Error message to display if there is a problem
 
         Returns:
             str: Path of downloaded file
-
         """
-        path = self.get_path_for_url(url, folder, filename, path, overwrite)
         f = None
         try:
             f = open(path, "wb")
@@ -357,19 +344,50 @@ class Download(BaseDownload):
                     f.flush()
             return f.name
         except Exception as e:
-            raise DownloadError(
-                f"Download of {url} failed in retrieval of stream!"
-            ) from e
+            raise DownloadError(errormsg) from e
         finally:
             if f:
                 f.close()
 
+    def stream_file(
+        self,
+        url: str,
+        folder: Optional[str] = None,
+        filename: Optional[str] = None,
+        path: Optional[str] = None,
+        overwrite: bool = False,
+        keep: bool = False,
+    ) -> str:
+        """Stream file from url and store in provided folder or temporary
+        folder if no folder supplied. Must call setup method first.
+
+        Args:
+            url (str): URL or path to download
+            folder (Optional[str]): Folder to download it to. Defaults to None (temporary folder).
+            filename (Optional[str]): Filename to use for downloaded file. Defaults to None (derive from the url).
+            path (Optional[str]): Full path to use for downloaded file. Defaults to None (use folder and filename).
+            overwrite (bool): Whether to overwrite existing file. Defaults to False.
+            keep (bool): Whether to keep already downloaded file. Defaults to False.
+
+        Returns:
+            str: Path of downloaded file
+        """
+        path = self.get_path_for_url(
+            url, folder, filename, path, overwrite, keep
+        )
+        if keep and exists(path):
+            return path
+        return self.stream_path(
+            path, f"Download of {url} failed in retrieval of stream!"
+        )
+
     def download_file(
         self,
         url: str,
         **kwargs: Any,
     ) -> str:
-        """Download file from url and store in provided folder or temporary folder if no folder supplied
+        """Download file from url and store in provided folder or temporary
+        folder if no folder supplied.
 
         Args:
             url (str): URL or path to download
@@ -378,6 +396,7 @@ class Download(BaseDownload):
             filename (str): Filename to use for downloaded file. Defaults to deriving from url.
             path (str): Full path to use for downloaded file instead of folder and filename.
             overwrite (bool): Whether to overwrite existing file. Defaults to False.
+            keep (bool): Whether to keep already downloaded file. Defaults to False.
             post (bool): Whether to use POST instead of GET. Defaults to False.
             parameters (Dict): Parameters to pass. Defaults to None.
             timeout (float): Timeout for connecting to URL. Defaults to None (no timeout).
@@ -386,8 +405,17 @@ class Download(BaseDownload):
 
         Returns:
             str: Path of downloaded file
-
         """
+        folder = kwargs.get("folder")
+        filename = kwargs.get("filename")
+        path = kwargs.get("path")
+        overwrite = kwargs.get("overwrite", False)
+        keep = kwargs.get("keep", False)
+        path = self.get_path_for_url(
+            url, folder, filename, path, overwrite, keep
+        )
+        if keep and exists(path):
+            return path
         self.setup(
             url,
             stream=True,
@@ -397,16 +425,12 @@ class Download(BaseDownload):
             headers=kwargs.get("headers"),
             encoding=kwargs.get("encoding"),
         )
-        return self.stream_file(
-            url,
-            folder=kwargs.get("folder"),
-            filename=kwargs.get("filename"),
-            path=kwargs.get("path"),
-            overwrite=kwargs.get("overwrite", False),
+        return self.stream_path(
+            path, f"Download of {url} failed in retrieval of stream!"
         )
 
     def download(self, url: str, **kwargs: Any) -> requests.Response:
-        """Download url
+        """Download url.
 
         Args:
             url (str): URL or path to download
@@ -419,7 +443,6 @@ class Download(BaseDownload):
 
         Returns:
             requests.Response: Response
-
         """
         return self.setup(
             url,
@@ -432,65 +455,59 @@ class Download(BaseDownload):
         )
 
     def get_header(self, header: str) -> Any:
-        """Get a particular response header of download
+        """Get a particular response header of download.
 
         Args:
             header (str): Header for which to get value
 
         Returns:
             Any: Response header's value
-
         """
         return self.response.headers.get(header)
 
     def get_headers(self) -> Any:
-        """Get response headers of download
+        """Get response headers of download.
 
         Returns:
             Any: Response headers
-
         """
         return self.response.headers
 
     def get_status(self) -> int:
-        """Get response status code
+        """Get response status code.
 
         Returns:
             int: Response status code
-
         """
         return self.response.status_code
 
     def get_text(self) -> str:
-        """Get text content of download
+        """Get text content of download.
 
         Returns:
             str: Text content of download
-
         """
         return self.response.text
 
     def get_yaml(self) -> Any:
-        """Get YAML content of download
+        """Get YAML content of download.
 
         Returns:
             Any: YAML content of download
-
         """
         with YAML() as yaml:
             return yaml.load(self.response.text)
 
     def get_json(self) -> Any:
-        """Get JSON content of download
+        """Get JSON content of download.
 
         Returns:
             Any: JSON content of download
-
         """
         return self.response.json()
 
     def download_text(self, url: str, **kwargs: Any) -> str:
-        """Download url as text
+        """Download url as text.
 
         Args:
             url (str): URL or path to download
@@ -503,13 +520,12 @@ class Download(BaseDownload):
 
         Returns:
             str: Text content of download
-
         """
         self.download(url, **kwargs)
         return self.get_text()
 
     def download_yaml(self, url: str, **kwargs: Any) -> Any:
-        """Download url as YAML
+        """Download url as YAML.
 
         Args:
             url (str): URL or path to download
@@ -522,13 +538,12 @@ class Download(BaseDownload):
 
         Returns:
             str: YAML content of download
-
         """
         self.download(url, **kwargs)
         return self.get_yaml()
 
     def download_json(self, url: str, **kwargs: Any) -> Any:
-        """Download url as JSON
+        """Download url as JSON.
 
         Args:
             url (str): URL or path to download
@@ -541,7 +556,6 @@ class Download(BaseDownload):
 
         Returns:
             str: JSON content of download
-
         """
         self.download(url, **kwargs)
         return self.get_json()
@@ -553,7 +567,7 @@ class Download(BaseDownload):
         infer_types: bool = False,
         **kwargs: Any,
     ) -> frictionless.Resource:
-        """Get Frictionless Resource
+        """Get Frictionless Resource.
 
         Args:
             url (str): URL or path to download
@@ -583,7 +597,6 @@ class Download(BaseDownload):
 
         Returns:
             frictionless.Resource: frictionless Resource object
-
         """
         self.close_response()
         try:
@@ -608,15 +621,16 @@ class Download(BaseDownload):
         ] = None,
         **kwargs: Any,
     ) -> Tuple[List[str], Iterator[ListDict]]:
-        """Returns header of tabular file pointed to by url and an iterator where each
-        row is returned as a list or dictionary depending on the dict_form argument. The
-        headers argument is either a row number or list of row numbers (in case of
-        multi-line headers) to be considered as headers (rows start counting at 1), or
-        the actual headers defined as a list of strings. It defaults to 1 and cannot be
-        None. The dict_form arguments specifies if each row should be returned as a
-        dictionary or a list, defaulting to a list.
+        """Returns header of tabular file pointed to by url and an iterator
+        where each row is returned as a list or dictionary depending on the
+        dict_form argument. The headers argument is either a row number or list
+        of row numbers (in case of multi-line headers) to be considered as
+        headers (rows start counting at 1), or the actual headers defined as a
+        list of strings. It defaults to 1 and cannot be None. The dict_form
+        arguments specifies if each row should be returned as a dictionary or a
+        list, defaulting to a list.
 
-        Optionally, headers can be inserted at specific positions. This is achieved
+        Optionally, headers can be inserted at specific positions. This is achieved blah blah blah blah blah blah
         using the header_insertions argument. If supplied, it is a list of tuples of the
         form (position, header) to be inserted. A function is called for each row. If
         supplied, it takes as arguments: headers (prior to any insertions) and row
@@ -653,7 +667,6 @@ class Download(BaseDownload):
 
         Returns:
             Tuple[List[str],Iterator[ListDict]]: Tuple (headers, iterator where each row is a list or dictionary)
-
         """
         if headers is None:
             raise DownloadError("Argument headers cannot be None!")
@@ -706,8 +719,9 @@ class Download(BaseDownload):
         ] = None,
         **kwargs: Any,
     ) -> Tuple[List[str], Iterator[ListDict]]:
-        """Returns header of tabular file(s) pointed to by url and an iterator where
-        each row is returned as a list or dictionary depending on the dict_rows argument.
+        """Returns header of tabular file(s) pointed to by url and an iterator
+        where each row is returned as a list or dictionary depending on the
+        dict_rows argument.
 
         When a list of urls is supplied (in url), then the has_hxl flag indicates if the
         files are HXLated so that the HXL row is only included from the first file.
@@ -755,7 +769,6 @@ class Download(BaseDownload):
 
         Returns:
             Tuple[List[str],Iterator[ListDict]]: Tuple (headers, iterator where each row is a list or dictionary)
-
         """
         if isinstance(url, list):
             is_list = True
@@ -779,8 +792,7 @@ class Download(BaseDownload):
             return outheaders, iterator1
 
         def make_iterator():
-            for row in iterator1:
-                yield row
+            yield from iterator1
             for url in urls[1:]:
                 temp_kwargs = deepcopy(orig_kwargs)
                 _, iterator = self._get_tabular_rows(
@@ -796,8 +808,7 @@ class Download(BaseDownload):
                 )
                 if has_hxl:
                     next(iterator)
-                for row in iterator:
-                    yield row
+                yield from iterator
 
         return outheaders, make_iterator()
 
@@ -815,7 +826,8 @@ class Download(BaseDownload):
         ] = None,
         **kwargs: Any,
     ) -> Tuple[List[str], Iterator[List]]:
-        """Returns headers and an iterator where each row is returned as a list.
+        """Returns headers and an iterator where each row is returned as a
+        list.
 
         When a list of urls is supplied (in url), then the has_hxl flag indicates if the
         files are HXLated so that the HXL row is only included from the first file.
@@ -861,7 +873,6 @@ class Download(BaseDownload):
 
         Returns:
             Tuple[List[str],Iterator[List]]: Tuple (headers, iterator where each row is a list)
-
         """
 
         headers, iterator = self.get_tabular_rows(
@@ -891,7 +902,8 @@ class Download(BaseDownload):
         ] = None,
         **kwargs: Any,
     ) -> Tuple[List[str], Iterator[Dict]]:
-        """Returns headers and an iterator where each row is returned as a dictionary.
+        """Returns headers and an iterator where each row is returned as a
+        dictionary.
 
         When a list of urls is supplied (in url), then the has_hxl flag indicates if the
         files are HXLated so that the HXL row is only included from the first file.
@@ -936,7 +948,6 @@ class Download(BaseDownload):
 
         Returns:
             Tuple[List[str], Iterator[Dict]]: Tuple (headers, iterator where each row is a dictionary)
-
         """
 
         headers, iterator = self.get_tabular_rows(
@@ -967,8 +978,8 @@ class Download(BaseDownload):
         ] = None,
         **kwargs: Any,
     ) -> Dict:
-        """Download 2 column csv from url and return a dictionary of keys (first column)
-        and values (second column).
+        """Download 2 column csv from url and return a dictionary of keys
+        (first column) and values (second column).
 
         When a list of urls is supplied (in url), then the has_hxl flag indicates if the
         files are HXLated so that the HXL row is only included from the first file.
@@ -1015,9 +1026,8 @@ class Download(BaseDownload):
 
         Returns:
             Dict: Dictionary keys (first column) and values (second column)
-
         """
-        output_dict = dict()
+        output_dict = {}
         _, rows = self.get_tabular_rows_as_list(
             url,
             has_hxl,
@@ -1049,9 +1059,9 @@ class Download(BaseDownload):
         ] = None,
         **kwargs: Any,
     ) -> Dict[str, Dict]:
-        """Download multicolumn csv from url and return dictionary where keys are first
-        column and values are dictionaries with keys from column headers and values from
-        columns beneath.
+        """Download multicolumn csv from url and return dictionary where keys
+        are first column and values are dictionaries with keys from column
+        headers and values from columns beneath.
 
         When a list of urls is supplied (in url), then the has_hxl flag indicates if the
         files are HXLated so that the HXL row is only included from the first file.
@@ -1098,7 +1108,6 @@ class Download(BaseDownload):
         Returns:
             Dict[str,Dict]: Dictionary where keys are first column and values are dictionaries with keys from column
             headers and values from columns beneath
-
         """
         headers, iterator = self.get_tabular_rows_as_dict(
             url,
@@ -1110,16 +1119,15 @@ class Download(BaseDownload):
             row_function,
             **kwargs,
         )
-        output_dict = dict()
+        output_dict = {}
         key_header = headers[keycolumn - 1]
         for row in iterator:
             first_val = row[key_header]
-            output_dict[first_val] = dict()
+            output_dict[first_val] = {}
             for header in row:
                 if header == key_header:
                     continue
-                else:
-                    output_dict[first_val][header] = row[header]
+                output_dict[first_val][header] = row[header]
         return output_dict
 
     def download_tabular_cols_as_dicts(
@@ -1136,9 +1144,9 @@ class Download(BaseDownload):
         ] = None,
         **kwargs: Any,
     ) -> Dict[str, Dict]:
-        """Download multicolumn csv from url and return dictionary where keys are header
-        names and values are dictionaries with keys from first column and values from
-        other columns.
+        """Download multicolumn csv from url and return dictionary where keys
+        are header names and values are dictionaries with keys from first
+        column and values from other columns.
 
         When a list of urls is supplied (in url), then the has_hxl flag indicates if the
         files are HXLated so that the HXL row is only included from the first file.
@@ -1185,7 +1193,6 @@ class Download(BaseDownload):
         Returns:
             Dict[str,Dict]: Dictionary where keys are header names and values are dictionaries with keys from first column
             and values from other columns
-
         """
         headers, iterator = self.get_tabular_rows_as_dict(
             url,
@@ -1197,12 +1204,12 @@ class Download(BaseDownload):
             row_function,
             **kwargs,
         )
-        output_dict = dict()
+        output_dict = {}
         key_header = headers[keycolumn - 1]
         for header in headers:
             if header == key_header:
                 continue
-            output_dict[header] = dict()
+            output_dict[header] = {}
         for row in iterator:
             for header in row:
                 if header == key_header:
@@ -1212,16 +1219,15 @@ class Download(BaseDownload):
 
     @staticmethod
     def get_column_positions(headers: ListTuple[str]) -> Dict[str, int]:
-        """Get mapping of headers to column positions
+        """Get mapping of headers to column positions.
 
         Args:
             headers (ListTuple[str]): List of headers
 
         Returns:
             Dict[str,int]: Dictionary where keys are header names and values are header positions
-
         """
-        columnpositions = dict()
+        columnpositions = {}
         for i, header in enumerate(headers):
             columnpositions[header] = i
         return columnpositions
@@ -1240,10 +1246,11 @@ class Download(BaseDownload):
     ) -> None:
         """Generate downloaders. Requires either global user agent to be set or
         appropriate user agent parameter(s) to be completed. The custom_configs
-        dictionary is a mapping from name to a dictionary of custom configuration
-        parameters that is added to the underlying session's params or headers. It can
-        have keys that correspond to the input arguments of Download's constructor
-        __init__ (or the other arguments of this method).
+        dictionary is a mapping from name to a dictionary of custom
+        configuration parameters that is added to the underlying session's
+        params or headers. It can have keys that correspond to the input
+        arguments of Download's constructor __init__ (or the other arguments of
+        this method).
 
         Args:
             custom_configs (Dict[str, Dict]): Optional dictionary of custom configurations.
@@ -1283,8 +1290,8 @@ class Download(BaseDownload):
 
     @classmethod
     def get_downloader(cls, name: Optional[str] = None) -> "Download":
-        """Get a generated downloader given a name. If name is not supplied, the default
-        one will be returned.
+        """Get a generated downloader given a name. If name is not supplied,
+        the default one will be returned.
 
         Args:
             name (Optional[str]): Name of downloader. Defaults to None (get default).
