@@ -1,4 +1,4 @@
-"""Session utilities for urls"""
+"""Session utilities for urls."""
 import logging
 import os
 from typing import Any, Optional
@@ -25,10 +25,12 @@ def get_session(
     user_agent_lookup: Optional[str] = None,
     use_env: bool = True,
     fail_on_missing_file: bool = True,
+    verify: bool = True,
     **kwargs: Any,
 ) -> requests.Session:
-    """Set up and return Session object that is set up with retrying. Requires either global user agent to be set or
-    appropriate user agent parameter(s) to be completed. If the EXTRA_PARAMS or BASIC_AUTH environment variable is
+    """Set up and return Session object that is set up with retrying. Requires
+    either global user agent to be set or appropriate user agent parameter(s)
+    to be completed. If the EXTRA_PARAMS or BASIC_AUTH environment variable is
     supplied, the extra_params* parameters will be ignored.
 
     Args:
@@ -37,6 +39,7 @@ def get_session(
         user_agent_lookup (Optional[str]): Lookup key for YAML. Ignored if user_agent supplied.
         use_env (bool): Whether to read environment variables. Defaults to True.
         fail_on_missing_file (bool): Raise an exception if any specified configuration files are missing. Defaults to True.
+        verify (bool): Whether to verify SSL certificates. Defaults to True.
         **kwargs: See below
         auth (Tuple[str, str]): Authorisation information in tuple form (user, pass) OR
         basic_auth (str): Authorisation information in basic auth string form (Basic xxxxxxxxxxxxxxxx) OR
@@ -50,7 +53,7 @@ def get_session(
         allowed_methods (ListTuple[str]): HTTP methods for which to force retry. Defaults to ("HEAD", "TRACE", "GET", "PUT", "OPTIONS", "DELETE").
     """
     s = requests.Session()
-
+    s.verify = verify
     ua = kwargs.get("full_agent")
     if not ua:
         ua = UserAgent.get(
@@ -58,7 +61,7 @@ def get_session(
         )
     s.headers["User-Agent"] = ua
 
-    auths_found = list()
+    auths_found = []
     headers = kwargs.get("headers")
     if headers is not None:
         s.headers.update(headers)
@@ -76,7 +79,7 @@ def get_session(
         extra_params = os.getenv("EXTRA_PARAMS")
         if extra_params:
             if "=" in extra_params:
-                extra_params_dict = dict()
+                extra_params_dict = {}
                 logger.info(
                     "Loading extra parameters from environment variable"
                 )
